import Vue from 'vue'
import Router from 'vue-router'
import store from '../store/store'
import Login from '@/components/Login'
import StudentPage from '@/components/student/StudentPage'
import StudentLayout from '@/components/student/StudentLayout'
import SubmissionDetail from '@/components/student/SubmissionDetail'
import ReviewerPage from '@/components/reviewer/ReviewerPage'
import StudentListOverview from '@/components/reviewer/StudentListOverview'
import BaseLayout from '@/components/base/BaseLayout'
import AnnotatedSubmission from '@/components/submission_notes/AnnotatedSubmission'

Vue.use(Router)

const router = new Router({
  routes: [
    {
      path: '/',
      name: 'grady-login',
      component: Login
    },
    {
      path: '/student/',
      component: StudentLayout,
      children: [
        {
          path: '',
          component: StudentPage
        },
        {
          path: 'submission/:id',
          component: SubmissionDetail
        }
      ]

    },
    {
      path: '/reviewer/',
      name: 'reviewer-page',
      component: ReviewerPage
    },
    {
      path: 'reviewer/student-overview/',
      name: 'student-overview',
      component: StudentListOverview
    },
    {
      path: '/base/',
      name: 'base-layout',
      component: BaseLayout
    },
    {
      path: '/notes/',
      name: 'annotated-submission',
      component: AnnotatedSubmission
    }
  ]
})

router.beforeEach((to, from, next) => {
  if (from.path === '/') {
    next()
  } else {
    const now = new Date()
    if (now - store.state.logInTime > store.state.jwtTimeDelta * 1000) {
      store.dispatch('logout').then(() => next('/'))
    } else {
      store.dispatch('refreshJWTToken')
      next()
    }
  }
})

export default router
