import Vuex from 'vuex'
import Vue from 'vue'
import ax from './api'

import gradySays from './grady_speak'
import submissionNotes from './modules/submission-notes'
import studentPage from './modules/student-page'

Vue.use(Vuex)

const store = new Vuex.Store({
  modules: {
    submissionNotes,
    studentPage
  },
  state: {
    token: '',
    loggedIn: false,
    logInTime: {},
    username: '',
    jwtTimeDelta: 0,
    userRole: '',
    error: '',
    examInstance: ''
  },
  getters: {
    gradySpeak: () => {
      return gradySays[Math.floor(Math.random() * gradySays.length)]
    }
  },
  mutations: {
    'API_FAIL': function (state, error) {
      state.error = error
    },
    'SET_JWT_TOKEN': function (state, token) {
      state.token = token
      state.logInTime = new Date()
      ax.defaults.headers.common['Authorization'] = 'JWT ' + token
    },
    'SET_JWT_TIME_DELTA': function (state, timeDelta) {
      state.jwtTimeDelta = timeDelta
    },
    'LOGIN': function (state, username) {
      state.loggedIn = true
      state.username = username
    },
    'LOGOUT': function (state) {
      state.loggedIn = false
    },
    'SET_USER_ROLE': function (state, userRole) {
      state.userRole = userRole
    },
    'SET_EXAM_INSTANCE': function (state, examInstance) {
      state.examInstance = examInstance
    }
  },
  actions: {
    async getJWTToken (context, credentials) {
      try {
        const response = await ax.post('api-token-auth/', credentials)
        context.commit('LOGIN', credentials.username)
        context.commit('SET_JWT_TOKEN', response.data.token)
      } catch (error) {
        if (error.response) {
          const errorMsg = 'Unable to log in with provided credentials.'
          context.commit('API_FAIL', errorMsg)
          throw errorMsg
        } else {
          const errorMsg = 'Cannot reach server.'
          context.commit('API_FAIL', errorMsg)
          throw errorMsg
        }
      }
    },
    refreshJWTToken (context) {
      ax.post('/api-token-refresh/', {token: context.state.token}).then(response => {
        context.commit('SET_JWT_TOKEN', response.data.token)
      })
    },
    getJWTTimeDelta (context) {
      ax.get('api/jwt-time-delta/').then(response => {
        context.commit('SET_JWT_TIME_DELTA', response.data.timeDelta)
      })
    },
    getUserRole (context) {
      ax.get('api/user-role/').then(response => context.commit('SET_USER_ROLE', response.data.role))
    },
    getExamModule (context) {
      ax.get('api/exam-module/').then(response => context.commit('SET_EXAM_INSTANCE', response.data.exam))
    },
    logout (store) {
      store.commit('LOGOUT')
      store.commit('SET_JWT_TOKEN', '')
    }
  }
})

export default store
